#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2024-07-06
# File  : vsphere_topo/rulesets/packages.py

# removed PARAM_ADD_DUMMY_TOPOLOGIES and PARAM_DONT_ADD_VC_AS_VM

from collections.abc import Mapping, Sequence

from cmk.rulesets.v1 import Help, Label, Message, Title
from cmk.rulesets.v1.form_specs import (
    CascadingSingleChoice,
    CascadingSingleChoiceElement,
    DefaultValue,
    DictElement,
    Dictionary,
    FixedValue,
    List,
    SingleChoice,
    SingleChoiceElement,
    String,
)
from cmk.rulesets.v1.form_specs.validators import LengthInRange, MatchRegex
from cmk.rulesets.v1.rule_specs import CheckParameters, HostCondition, Topic
from cmk_addons.plugins.vsphere_topo.constants import (
    Emblem,
    Param,
    Picture,
    ICON_VCENTER,
    RULE_SET_NAME_VSPHERE_TOPO,
)

adjust_names_elements: Mapping[str, DictElement] = {
    Param.KEEP_DOMAIN: DictElement(
        parameter_form=FixedValue(
            title=Title('Keep domain name'),
            label=Label('The domain name will not be cut'),
            value=True,
        )),
    Param.CHANGE_CASE: DictElement(
        parameter_form=SingleChoice(
            title=Title('Change case'),
            elements=[
                SingleChoiceElement(name=Param.UPPER, title=Title('All upper case')),
                SingleChoiceElement(name=Param.LOWER, title=Title('All lower case')),
                SingleChoiceElement(name=Param.NO_CHANGE, title=Title('Don\'t change case'))
            ],
            prefill=DefaultValue(Param.NO_CHANGE)
        )),
    Param.PREFIX: DictElement(
        parameter_form=String(
            title=Title('Name prefix'),
            custom_validate=(LengthInRange(min_value=1),)
        )),
    Param.SUFFIX: DictElement(
        parameter_form=String(
            title=Title('Name suffix'),
            custom_validate=(LengthInRange(min_value=1),)
        )),
}


def get_emblem_element(default_emblem: str, picture_type: str) -> Sequence[CascadingSingleChoiceElement]:
    return [
        CascadingSingleChoiceElement(
            name=Param.NO_EMBLEM,
            title=Title(f'No custom {picture_type}'),
            parameter_form=FixedValue(
                value=True,
                label=Label(f'No custom {picture_type} will be used')
            )),
        CascadingSingleChoiceElement(
            name=Param.DEFAULT_EMBLEM,
            title=Title(f'Use default {picture_type}'),
            parameter_form=FixedValue(
                value=True,
                label=Label(f'"{default_emblem}" will be used as {picture_type}')
            )),
        CascadingSingleChoiceElement(
            name=Param.CUSTOM_EMBLEM,
            title=Title(f'Use custom {picture_type}'),
            parameter_form=String(
                custom_validate=(LengthInRange(min_value=1),),
                prefill=DefaultValue(default_emblem),
            ))
    ]


def _parameter_form() -> Dictionary:
    return Dictionary(
        elements={
            Param.HOST_SYSTEMS: DictElement(
                parameter_form=Dictionary(
                    title=Title('Adjust ESXi host names'),
                    elements=adjust_names_elements
                )),
            Param.VM_NAMES: DictElement(
                parameter_form=Dictionary(
                    title=Title('Adjust VM names'),
                    elements=adjust_names_elements
                )),
            Param.VCENTER: DictElement(
                parameter_form=CascadingSingleChoice(
                    title=Title('vCenter Icon'),
                    elements=get_emblem_element(ICON_VCENTER, Picture.TYPE_ICON),
                    prefill=DefaultValue(Param.DEFAULT_EMBLEM),
                    help_text=Help(
                        'Here you can change the icon for the vCenter object. If you use the built-in icons prefix '
                        'the name with "icon_"'
                    ),
                )),
            Param.CLUSTER: DictElement(
                parameter_form=CascadingSingleChoice(
                    title=Title('Cluster emblem'),
                    elements=get_emblem_element(Emblem.CLUSTER, Picture.TYPE_EMBLEM),
                    prefill=DefaultValue(Param.DEFAULT_EMBLEM),
                    help_text=Help(
                        'Here you can change the picture for the cluster. If you use the built-in icons prefix '
                        'the name with "icon_"'
                    ),
                )),
            Param.DATA_CENTER: DictElement(
                parameter_form=CascadingSingleChoice(
                    title=Title('Datacenter emblem'),
                    elements=get_emblem_element(Emblem.DATA_CENTER, Picture.TYPE_EMBLEM),
                    prefill=DefaultValue(Param.DEFAULT_EMBLEM),
                    help_text=Help(
                        'Here you can change the picture for the datacenter. If you use the built-in icons prefix '
                        'the name with "icon_"'
                    ),
                )),
            Param.DATA_STORE: DictElement(
                parameter_form=CascadingSingleChoice(
                    title=Title('Datastore emblem'),
                    elements=get_emblem_element(Emblem.DATA_STORE, Picture.TYPE_EMBLEM),
                    prefill=DefaultValue(Param.DEFAULT_EMBLEM),
                    help_text=Help(
                        'Here you can change the picture for the datastore. If you use the built-in icons prefix '
                        'the name with "icon_"'
                    ),
                )),
            Param.DATA_STORE_AS_SERVICE: DictElement(
                parameter_form=FixedValue(
                    title=Title('Add data store service'),
                    label=Label('enabled'),
                    help_text=Help(
                        'This option adds the datastore services from the vCenter object to the topology. '
                        'Pro: more insight. Con: cluttered topology.'
                    ),
                    value=True
                )),
            Param.MAKE_DEFAULT: DictElement(
                parameter_form=FixedValue(
                    title=Title('Make default'),
                    label=Label('This will be the default topology'),
                    help_text=Help(
                        'Makes the vSphere topology the default topology. If there no default topology, the vSphere '
                        'topology becomes always the default.'
                    ),
                    value=True
                )),
            Param.IGNORE_POWERED_OFF_VMS: DictElement(
                parameter_form=FixedValue(
                    title=Title('Ignore powered off VMs'),
                    label=Label('enabled'),
                    help_text=Help(
                        'Powered off VMs will not be added to the topology'
                    ),
                    value=True,
                )),
            Param.IGNORE_VM_NAME_REGEX: DictElement(
                parameter_form=List(
                    title=Title('Ignore VMs by name (regex)'),
                    element_template=String(),
                    help_text=Help(
                        'The regex strings will be applied after the upper/lower case and prefix '
                        'change to the VM names. If there more than one regex string the will be "OR" connected.'
                    ),
                )),
            Param.MAP_NAMES: DictElement(
                parameter_form=List(
                    title=Title('Map vSphere names to CMK names'),
                    element_template=Dictionary(
                        elements={
                            Param.MAP_NAME_VSPHERE: DictElement(
                                parameter_form=String(
                                    title=Title('vSphere name'),
                                ),
                                required=True,
                            ),
                            Param.MAP_NAME_CMK: DictElement(
                                parameter_form=String(
                                    title=Title('Checkmk name'),
                                    custom_validate=(
                                        MatchRegex(
                                            regex=r'^[0-9a-zA-Z\.\-\_]+$',
                                            error_msg=Message(
                                                'Your input does not match the required format. '
                                                'Only "0-9a-zA-Z_-." are allowed.'
                                            )
                                        ),)
                                ),
                                required=True,
                            )
                        }
                    ),
                    help_text=Help(''),
                )),
        }
    )


rule_spec_vsphere_topo = CheckParameters(
    name=RULE_SET_NAME_VSPHERE_TOPO,
    topic=Topic.APPLICATIONS,
    parameter_form=_parameter_form,
    title=Title('vSphere Topology'),
    condition=HostCondition(),
)
